grenswaarden = {    
    # Test 	Unit	Drempel_onder	Grens_onder	Drempel_boven	Grens_boven	Lambda function						
    "Temperatuur":	["C",	None,	None,	25,	25, 	lambda sol: sol.temperature],
    "Zuurstof":	["mg/l",	4,	2,	None,	None,	lambda sol: sol.molality('O2') * 32e3],
    "pH":	["pH",	7.7,	7,	8.3,	9.5,	lambda sol: sol.pH],
    "AgCO2":	["mg/l",	None,	None,	2.2,	2.2,	lambda sol: sol.aggco2 * 44],
    "EGV":	["mS/m",	None,	None,	80,	125,	lambda sol: sol.sc * 0.1],
    "HCO3":	["mg/l",	90,	60,	None,	None,	lambda sol: sol.total('HCO3') * 61e3],
    "Cl":	["mg/l",	None,	None,	100,	150,	lambda sol: sol.total('Cl') * 35.5e3],
    "SO4":	["mg SO4 / l",	None,	None,	100,	150,	lambda sol: sol.total('SO4') * 96.1e3],
    "NH4":	["mg NH4 / l",	None,	None,	0.05,	0.2,	lambda sol: sol.total('NH4') * 18e3],
    "NO2":	["mg NO2 / l",	None,	None,	0.03,	0.05,	lambda sol: sol.total('NO2') * 46e3],
    "NO3":	["mg NO3 / l",	None,	None,	25,	50,	lambda sol: sol.total('NO3') * 63e3],
    "CN":	["ug/l",	None,	None,	10,	50,	lambda sol: sol.total('CN') * 26e6],
    "F":	["mg/l",	None,	None,	1,	1,	lambda sol: sol.total_element('F') * 19e3],
    "TAC":	["mg/l",	24,	24,	None,	None,	lambda sol: sol.total('CO2') * 44e3 + sol.total('HCO3') * 61e3 + sol.total('CO3') * 60e3],
    "TACC90":	["mmol/l",	None,	None,	0.4,	0.6,	lambda sol: sol.tacc90],
    "Natrium":	["mg/l",	None,	None,	100,	150,	lambda sol: sol.total_element('Na') * 23e3],
    "Hardheid":	["mmol/l",	1,	1,	1.43,	2,	lambda sol: sol.total('Ca')*1e3 + sol.total('Mg')*1e3],
    "Ijzer":	["mg/l",	None,	None,	0.05,	0.1,	lambda sol: sol.total_element('Fe')*55.8e3],
    "Mangaan":	["mg/l",	None,	None,	0.01,	0.05,	lambda sol: sol.total_element('Mn')*54.9e3],
    "Aluminium":	["ug/l",	None,	None,	30,	30,	lambda sol: sol.total_element('Al')*27e6],
    "Antimoon":	["ug/l",	None,	None,	5,	5,	lambda sol: sol.total_element('Sb')*122e6],
    "Arseen":	["ug/l",	None,	None,	1,	10,	lambda sol: sol.total_element('As')*74.9e6],
    "Barium":	["ug/l",	None,	None,	100,	500,	lambda sol: sol.total_element('Ba')*137e6],
    "Boor":	["ug/l",	None,	None,	300,	500,	lambda sol: sol.total_element('B')*10.8e6],
    "Cadmium":	["ug/l",	None,	None,	1,	5,	lambda sol: sol.total_element('Cd')*112e6],
    "Chroom":	["ug/l",	None,	None,	10,	50,	lambda sol: sol.total_element('Cr')*52e6],
    "Koper":	["ug/l",	None,	None,	2000,	2000,	lambda sol: sol.total_element('Cu')*63.5e6],
    "Kwik":	["ug/l",	None,	None,	0.2,	0.5,	lambda sol: sol.total_element('Hg')*201e6],
    "Lood":	["ug/l",	None,	None,	10,	10,	lambda sol: sol.total_element('Pb')*207e6],
    "Nikkel":	["ug/l",	None,	None,	10,	20,	lambda sol: sol.total_element('Ni')*58.7e6],
    "Seleen":	["ug/l",	None,	None,	10,	10,	lambda sol: sol.total_element('Se')*79e6],
    "Zink":	["ug/l",	None,	None,	100,	100,	lambda sol: sol.total_element('Zn')*65.4e6]
}




from phreeqpython import PhreeqPython, Solution
from .chemistry import chemicals
from piwebapi import Element, Point

try:
    import influxdb
except:
    pass

lims_map = {
    "Al":(182,"AL"),
    "CH4":(226,"CH4"),
    "Ca":(144,"CA"),
    "Mg":(145,"MG"),
    "Cl":(164,"CL"),
    "CO2": (148, "KOOLD"),
    "EGV 20°C":(116,"GELEIDING"),
    "Fe":(146,"FE"),
    "Hardheid":(162,"HHTOT"),
    "HCO3":(150,"HCO3"),
    "K":(122,"K"),
    "Mn":(147,"MN"),
    "Na":(120,"NA"),
    "NH4":(166,"NH4"),
    "NO3":(118,"NO3"),
    "pH":(115,"PH"),
    "SO4":(715,"SO4"),
    "TOC":(405,"TOC")
}

#############################################
# Vitens phreeqpython extensions for PI A/F #
#############################################

def add_solution_af(self, path, attr_name="Waterkwaliteitsmetingen", postfix="(lab)", time='*', persistence=604800):
    """ Create a PhreeqPython solution using data from a PI/AF tag """
    if isinstance(path, str):
        attributes = Element(path, persistence=persistence).attributes[attr_name]
    else:
        attributes = path.attributes[attr_name]
    composition = {}
    charge = 0

    for name, element in attributes.children.items():
        name = name.replace(" "+postfix,"")
        value = element.get_value(time, persistence)[0]

        # skip emtpy or negative values
        if isinstance(value, float) and value < 0:
            continue

        if name in chemicals:
            mg = value if element.units == "milligram per liter" else value/1000
            #print name, mg, element.units
            mmol = mg / chemicals[name][0]
            if mmol > 0:
                composition[name] = mmol
                charge += mmol * chemicals[name][1]
        elif name == "DOC":
            composition["Toc"] = value
        # decouple methane
        elif name == "CH4":
            composition["Mtg"] = value / 1000

    if charge > 0:
        composition['Nmod'] = charge
    if charge < 0:
        composition['Pmod'] = -charge

    try:
        temperatuur = attributes["Temperatuur"]
    except:
        temperatuur = 10

    return self.add_solution_simple(composition, temperatuur)


PhreeqPython.add_solution_af = add_solution_af

def write_to_influx(self, path, attribute="Waterkwaliteit Softsensor"):
    if isinstance(path, str):
        element = Element(path)
    else:
        element = path

    attributes = element.attributes[attribute]

    influx_json = []
    
    for name, attr in attributes.items():
        name = name.replace(" (lab)", "")
        if name not in lims_map:
            continue
        lims = lims_map[name]

        if name in chemicals:
            mmol = self.total(name)
            mg = chemicals[name][0] * mmol
            value = mg if attr.units == "milligram per liter" else mg * 1000
        elif name == "pH": 
            value = self.pH
        elif name[:3] == "EGV":
            value = self.egv / 10
        elif name == "Hardheid":
            value = (self.total_element('Ca') + self.total_element('Mg'))
        elif name == "TOC":
            value = self.total_element('Toc')
        elif name == "CH4":
            value = self.total_element('Mtg') * 1000
        else:
            continue
        
        influx_json.append({
            "measurement": attributes["Monsterpuntcode"].value,
            "tags": {
                "testcode": lims[0],
                "analyte": lims[1]
                },
            "fields": {
                "value": value
                }
            })

    client = influxdb.InfluxDBClient(database='slimm')
    client.write_points(influx_json)

def write_to_af(self, path, postfix, tagfilter=None):

    if isinstance(path, str):
        element = Element(path)
    else:
        element = path

    attributes = element.attributes["Waterkwaliteitsmetingen"]

    for tagname, attribute in attributes.items():
        if "(ss)" in tagname:

            if tagfilter is not None and tagfilter not in tagname:
                continue

            name = tagname.split(" ")[0]

            if name in chemicals:
                mmol = self.total(name) * 1000
                mg = chemicals[name][0] * mmol
                value = mg if attribute.units == "milligram per liter" else mg * 1000
            elif name == "pH": 
                value = self.pH
            elif name == "EGV":
                value = self.egv / 10
            elif name == "Hardheid":
                value = (self.total_element('Ca') + self.total_element('Mg')) * 1000
            elif name == "TOC":
                value = self.total_element('Toc') * 1000
            else:
                continue
                #raise NotImplementedError(name)

            path = attribute.config_string
            try:
                pipath = path.split("?")[0] + "\\" + path.split("\\")[3].split("?")[0] + postfix
                point = Point(pipath)
                point.write_value(value)
            except:
                print("write failed")
                continue

Solution.write_to_af = write_to_af
Solution.write_to_influx = write_to_influx



def cijfer(waarde, grens_onder=None, drempel_onder=None, drempel_boven=None, grens_boven=None):
    cijfer = 0
    
    R = waarde
    M = grens_onder
    L = drempel_onder
    D = drempel_boven
    G = grens_boven
    
    # alleen bovengrenzen
    if drempel_boven and not drempel_onder:
        if R<=D and D is not G:
            cijfer = 10-2*R/D
        elif R<=D:
            cijfer = 10-4*R/D
        elif R<=G and D is not G:
            cijfer = 8-2*(R-D)/(G-D)
        elif D is not G:
            cijfer = 6-2*(R-G)/(G-D)
        else:
            cijfer = 6-4*(R-G)/D
    
    # alleen ondergrenzen
    elif drempel_onder and not drempel_boven:
        if R>=L and L is not M:
            cijfer = 8+2*(R-L)/(L-M)
        elif R>=L:
            cijfer = 6+6*(R-L)/L
        elif R>=M and L is not M:
            cijfer = 6+2*(R-M)/(L-M)
        else:
            cijfer = 6-6*(M-R)/M

    # onder en bovengrenzen
    else:
        A = D-(D-L)/2
        if R>G and G is not D:
            cijfer = 6-2*(R-G)/(G-D)
        elif R>=G:
            cijfer = 6-4*(R-G)/(G-A)
        elif R<=G and R>D:
            cijfer = 8-2*(R-D)/(G-D)
        elif R<=D and R>=A and G is not D:
            cijfer = 10-2*(R-A)/(D-A)
        elif R<=D and R>=A:
            cijfer = 10-4*(R-A)/(D-A)
        elif R<A and R>=L and L is not M:
            cijfer = 8+2*(R-L)/(A-L)
        elif R<A and R>=L:
            cijfer = 6+4*(R-L)/(A-L)
        elif R<L and R>=M and L is not M:
            cijfer = 6+2*(R-M)/(L-M)
        else:
            cijfer = 6-6*(M-R)/M
    
    if cijfer > 10:
        return 10
    elif cijfer < 0:
        return 0
    else:
        return float(cijfer)



try:
    import influxdb
    import piwebapi
except:
    pass


def write_softsensor(self, value):

    if'Monsterpuntcode' in self.children:
        puntcode = self.children['Monsterpuntcode'].value
    else:
        puntcode = piwebapi.Attribute(self.links['Parent']).attributes['Monsterpuntcode'].value

    if '_SS' not in puntcode:
        raise ValueError('Dit is geen softsensor')

    analyte = self.children['Analyte'].value
    testcode = self.children['Testcode'].value

    influx_json = []
    influx_json.append({
        "measurement": puntcode,
        "tags": {
            "testcode": testcode,
            "analyte": analyte
            },
        "fields": {
            "value": value
            }
        })

    client = influxdb.InfluxDBClient(database='slimm')
    client.write_points(influx_json)


piwebapi.Attribute.write_softsensor = write_softsensor



from phreeqpython import Solution
from .cijfer import cijfer
from .grenswaarden import grenswaarden

#######################################
# Vitens specific solution extensions #
#######################################

def si90(self, phase):

    tmp = self.copy()
    tmp.change_temperature(90)

    si = tmp.si(phase)

    tmp.forget()
    return si

def tacc(self,temperature=None):
    """ Calculate the Calcium Carbonate Precipitation Potential (CCPP)"""
    # create temporary solution
    tmp = self.copy()
    # raise temperature
    if temperature and temperature is not self.temperature:
        tmp.change_temperature(temperature)
    ca_pre = tmp.total_element('Ca')
    # use saturate instead of desaturate to allow dissolution in addition to precipitation
    tmp.equalize('Calcite',0.0)
    # calculate tacc
    tacc = ca_pre - tmp.total_element('Ca')
    # cleanup
    self.pp.remove_solutions([tmp.number])
    return tacc #mmol

@property
def aggco2(self):
    tacc = self.tacc(self.temperature)
    if tacc < 0:
        return abs(tacc)
    else:
        return 0

def score(self,test, precision=2):
    """ Returns the score for a Vitens grens- en drempelwaarden test """
    test_values = grenswaarden[test]
    # perform lambda function
    value = test_values[5](self)
    return round(cijfer(value,test_values[2],test_values[1],test_values[3],test_values[4]),precision)

@property
def scores(self):
    scores = {}
    for test in grenswaarden.keys():
        scores[test] = score(self,test)
    return scores

# vitens TACC90 calculation
@property
def tacc90(self):
    return self.tacc(90)

@property
def th(self):
    return (self.total_element('Ca')+self.total_element('Mg'))*1e3

# Vitens EGV calculation at 20 degrees
@property
def egv(self):
    tmp = self.copy()
    tmp.change_temperature(20)
    egv = tmp.sc
    tmp.forget()
    return egv

@property
def egv25(self):
    tmp = self.copy()
    tmp.change_temperature(25)
    egv = tmp.sc
    tmp.forget()
    return egv

# extend class
Solution.tacc = tacc
Solution.tacc90 = tacc90
Solution.aggco2 = aggco2
Solution.score = score
Solution.scores = scores
Solution.egv = egv
Solution.egv25 = egv25
Solution.si90 = si90
Solution.th = th



from setuptools import setup

setup(name='vitenstools',
      version='0.9.1',
      description='Vitens Specific Tools and Extensions for PhreeqPython',
      url='https://github.com/AbelHeinsbroek/vitens',
      author='Abel Heinsbroek',
      author_email='abel.heinsbroek@vitens.nl',
      license='Apache Licence 2.0',
      packages=['vitenstools'],
      zip_safe=False)
